@@ -1,10 +1,10 @@
-{-# LANGUAGE BangPatterns, ForeignFunctionInterface, MagicHash,
+{-# LANGUAGE BangPatterns, CPP, ForeignFunctionInterface, MagicHash,
     UnliftedFFITypes #-}
 -- |
 -- Module      : Data.Text.Encoding
--- Copyright   : (c) 2008, 2009 Tom Harper,
---               (c) 2009, 2010 Bryan O'Sullivan,
---               (c) 2009 Duncan Coutts
+-- Copyright   : (c) 2009, 2010, 2011 Bryan O'Sullivan,
+--               (c) 2009 Duncan Coutts,
+--               (c) 2008, 2009 Tom Harper
 --
 -- License     : BSD-style
 -- Maintainer  : bos@serpentine.com, rtomharper@googlemail.com,
@@ -48,7 +48,11 @@ module Data.Text.Encoding
     ) where
 
 import Control.Exception (evaluate, try)
+#if __GLASGOW_HASKELL__ >= 702
+import Control.Monad.ST.Unsafe (unsafeIOToST, unsafeSTToIO)
+#else
 import Control.Monad.ST (unsafeIOToST, unsafeSTToIO)
+#endif
 import Data.Bits ((.&.))
 import Data.ByteString as B
 import Data.ByteString.Internal as B
@@ -60,7 +64,7 @@ import Data.Word (Word8)
 import Foreign.C.Types (CSize)
 import Foreign.ForeignPtr (withForeignPtr)
 import Foreign.Marshal.Utils (with)
-import Foreign.Ptr (Ptr, plusPtr)
+import Foreign.Ptr (Ptr, minusPtr, plusPtr)
 import Foreign.Storable (peek, poke)
 import GHC.Base (MutableByteArray#)
 import System.IO.Unsafe (unsafePerformIO)
@@ -149,8 +153,9 @@ encodeUtf8 (Text arr off len) = unsafePerformIO $ do
    where
     loop n1 m1 ptr = go n1 m1
      where
+      offLen = off + len
       go !n !m
-        | n-off == len = return (PS fp 0 m)
+        | n == offLen = return (PS fp 0 m)
         | otherwise = do
             let poke8 k v = poke (ptr `plusPtr` k) (fromIntegral v :: Word8)
                 ensure k act
@@ -164,8 +169,19 @@ encodeUtf8 (Text arr off len) = unsafePerformIO $ do
                 {-# INLINE ensure #-}
             case A.unsafeIndex arr n of
              w| w <= 0x7F  -> ensure 1 $ do
-                  poke8 m w
-                  go (n+1) (m+1)
+                  poke (ptr `plusPtr` m) (fromIntegral w :: Word8)
+                  -- A single ASCII octet is likely to start a run of
+                  -- them.  We see better performance when we
+                  -- special-case this assumption.
+                  let end = ptr `plusPtr` size
+                      ascii !t !u
+                        | t == offLen || u == end || v >= 0x80 =
+                            go t (u `minusPtr` ptr)
+                        | otherwise = do
+                            poke u (fromIntegral v :: Word8)
+                            ascii (t+1) (u `plusPtr` 1)
+                        where v = A.unsafeIndex arr t
+                  ascii (n+1) (ptr `plusPtr` (m+1))
               | w <= 0x7FF -> ensure 2 $ do
                   poke8 m     $ (w `shiftR` 6) + 0xC0
                   poke8 (m+1) $ (w .&. 0x3f) + 0x80
