@@ -4,7 +4,7 @@
 -- Copyright   : (c) Bryan O'Sullivan 2009
 --
 -- License     : BSD-style
--- Maintainer  : bos@serpentine.com, rtharper@aftereternity.co.uk,
+-- Maintainer  : bos@serpentine.com, rtomharper@googlemail.com,
 --               duncan@haskell.org
 -- Stability   : experimental
 -- Portability : GHC
@@ -29,6 +29,7 @@ module Data.Text.Fusion.Common
     , init
     , null
     , lengthI
+    , compareLengthI
     , isSingleton
 
     -- * Transformations
@@ -99,8 +100,8 @@ module Data.Text.Fusion.Common
     ) where
 
 import Prelude (Bool(..), Char, Eq(..), Int, Integral, Maybe(..),
-                Ord(..), String, (.), ($), (+), (-), (*), (++), (&&),
-                fromIntegral, otherwise)
+                Ord(..), Ordering(..), String, (.), ($), (+), (-), (*), (++),
+                (&&), fromIntegral, otherwise)
 import qualified Data.List as L
 import qualified Prelude as P
 import Data.Int (Int64)
@@ -158,8 +159,8 @@ snoc (Stream next0 xs0 len) w = Stream next (J xs0) (len+1)
     next N = Done
 {-# INLINE [0] snoc #-}
 
-data E l r = L {-# UNPACK #-} !l
-           | R {-# UNPACK #-} !r
+data E l r = L !l
+           | R !r
 
 -- | /O(n)/ Appends one Stream to the other.
 append :: Stream Char -> Stream Char -> Stream Char
@@ -266,6 +267,25 @@ lengthI (Stream next s0 _len) = loop_length 0 s0
                            Yield _ s' -> loop_length (z + 1) s'
 {-# INLINE[0] lengthI #-}
 
+-- | /O(n)/ Compares the count of characters in a string to a number.
+-- Subject to fusion.
+--
+-- This function gives the same answer as comparing against the result
+-- of 'lengthI', but can short circuit if the count of characters is
+-- greater than the number, and hence be more efficient.
+compareLengthI :: Integral a => Stream Char -> a -> Ordering
+compareLengthI (Stream next s0 len) n = 
+    case exactly len of
+      Nothing -> loop_cmp 0 s0
+      Just i  -> compare (fromIntegral i) n
+    where
+      loop_cmp !z s  = case next s of
+                         Done       -> compare z n
+                         Skip    s' -> loop_cmp z s'
+                         Yield _ s' | z > n     -> GT
+                                    | otherwise -> loop_cmp (z + 1) s'
+{-# INLINE[0] compareLengthI #-}
+
 -- | /O(n)/ Indicate whether a string contains exactly one element.
 isSingleton :: Stream Char -> Bool
 isSingleton (Stream next s0 _len) = loop 0 s0
@@ -662,8 +682,8 @@ take n0 (Stream next0 s0 len) =
 {-# INLINE [0] take #-}
 
 -- | /O(n)/ drop n, applied to a stream, returns the suffix of the
--- stream of length @n@, or the empty stream if @n@ is greater than the
--- length of the stream.
+-- stream after the first @n@ characters, or the empty stream if @n@
+-- is greater than the length of the stream.
 drop :: Integral a => a -> Stream Char -> Stream Char
 drop n0 (Stream next0 s0 len) =
     Stream next (J n0 :*: s0) (len - fromIntegral (max 0 n0))
